@@ -8,7 +8,10 @@
 #==============================================================================
 import sys
 import os
-from .ModuleLink import toPyVuka as pyvuka
+try:
+    from .ModuleLink import toPyVuka as pyvuka
+except:
+    from ModuleLink import toPyVuka as pyvuka  # to run directly
 
 __author__ = "R. Paul Nobrega"
 __app_name__ = "PyVuka"
@@ -16,7 +19,7 @@ __description__ = "A General Purpose Global Data Analysis Package"
 __copyright__ = "Copyright 2017, www.BostonAutoLytics.com"
 __credits__ = "R. Paul Nobrega, Osman Bilsel, & David G. Lambright"
 __license__ = "GPL"
-__version__ = "0.1.0.8"
+__version__ = "0.1.0.81"
 __maintainer__ = "R. Paul Nobrega"
 __email__ = "Paul@BostonAutoLytics.com"
 __status__ = "Development"
